function xout = nonduplicate(xin)
% function xout = nonduplicate(xin)
% 
% written so that interp1 doesn't return error:
%		'The values of X should be distinct.'
% 
% does not sort data
% checks for duplicate values in xin 
% returns xout where all values are distinct
% 
% if duplicates found, separates values by small value (eps)
% 
% input xin must be 1D
% 
% useage:
%	x = nonduplicate(x);
% 
%	yi = interp1(nonduplicate(x),y,xi)
% 
% Example:
% interp1([1,1,2],[1,1,2],1.5)
% returns an error.
% 
% interp1(nonduplicate([1,1,2]),[1,1,2],1.5)
% works just fine.
% 
% written by Nathan Tomlin, nathan.a.tomlin@gmail.com
% v2: update 190321 after tip from Natalia Aziares Aguayo <n.aziares.a@gmail.com>
%		same as listed here: https://stackoverflow.com/questions/13407030/desort-a-vector-undo-a-sorting
% 
% should probably use consolidator instead


warning('you should use consolidator instead')

xin2 = xin(:);

% sort 1st, but keep track of indices to "unsort" at end
[xout,isort] = sort(xin2);
% figure; hold on; plot(xin), plot(xout,'r')
% figure; plot(xout(isort))
% figure; plot((isort))
% isort

dx = diff(xout);	% dx=0 means 2 values are the same
idupl = find(dx==0);	% indices of duplicates
if ~isempty(idupl)	% duplicates in xin - need to fix
	% look at duplicates to find how many duplicates there are
	% for each duplicate value
		% example: if	xin =	[1,1,1,1,1]
		% then want		xout =	[1,1+eps,1+2*eps,1+3*eps,1+4*eps]
		
	didupl = diff(idupl);	% ones mean 3 or more of the same value
	k = 1; 	
	multeps(1) = 1;
	for m = 1:numel(didupl)
		if didupl(m) == 1	% 3 or more of the same value
			k = k+1;
		else				% only 2 of same value OR new duplicate value
			k = 1;
		end
		multeps(m+1) = k;	% multiplication factor to increase duplicate value
	end
	
	multeps = multeps(:);

	% increment duplicate values so they are distinct
	xout(idupl+1) = xout(idupl+1) + eps(xout(idupl+1)) .* multeps;

	% "unsort"
	xout2(isort) = xout;
	xout = xout2;
else	% no duplicates in xin - done
	xout = xin;
end



